% responses.m
%
% Computes IRF during a credible disinflation program
clear all
clc
%% Variables and dimensions
%
%  A0 y(t) = A1 y(t-1) + B0 E_t y(t+1) + D0 e(t)
%
%  gives a solution of the form
%  
%  y(t) = Q y(t-1) + G e(t)
%
% Equations: 
%
% x(t) = (r_ss-ppi_ss)-(r(t)-Et pi(t+1))+Et x(t+1)+(1-omega)(1-rho_a)a(t)
%
% pi(t) = (1-beta)*ppi_ss+beta*Et pi(t+1)+psi x(t)-ehat(t))
%
% r(t) = (1-rho_r)*rss-(rho_pi)*ppi_ss-rho_g*g_ss+rho_r*r(t-1) +
%        + rho_pi*pi(t)+rho_g*g(t)+rho_x*x(t)+er(t)
%
% x(t) = yhat(t)-omega a(t)
%
% g(t) = g_ss + yhat(t) - yhat(t-1) + e_z(t)
% 
% a(t) = rho_a*a(t-1) + e_a(t)
% 
% e(t) = rho_e*e(t-1) + e_e(t)

% Order of Variables in z(t)
%   1     2      3     4      5     6      7     8
% r(t)  yhat(t) g(t)  a(t)   e(t)  x(t)   ppi(t) const - 8

r = 1; yhat = 2; g = 3; a = 4; e = 5; x = 6; ppi = 7; const = 8; 
n = 8;

% Order of Shocks in e(t)
%   1     2      3     4      
%  er(t) ea(t), ee(t), ez(t);
er = 1; ea =2; ee = 3; ez = 4;
% Dimension of et(t)
nshocks = 4;

%% Parameters 
% Monetary policy
rho_r = 0.9;
rho_pi = 0.4;
rho_g = 0.1;
rho_x = 0.05;
% % Households and firms 
alpha = 0.0; 
beta = 0.9975;
psi = 0.1;
omega = 0.1;
% % Exogenous processes
rho_a = 0.85;
rho_e = 0.85;
% % Steady state constants
g_ss = 0.005; % Steady state quarterly growth rate of output per capita
ppi_ss = 0.00125; % Steady state quarterly rate of price inflation
r_ss = ppi_ss + g_ss - log(beta); % Steady state nominal interest rate.
% % Standard Errors
sig_r = 0.0017;
sig_a = 0.0100;
sig_e = 0.0018;
sig_z = 0.0140;

% Disinflation parameters
Tm = 4;
Tmstar = 4+8;
lambda = 4^(1/(Tmstar-Tm)); % lambda^(Tmstar-Tm) = ppi_ss_f / ppi_ss; %  0.5 implies ppi_ss_f = 0.5 x ppi_ss                       
gamma = 1^(1/(Tmstar-Tm));  % rate of increase in rho_pi                       
                              
Tb = 50;
Tbstar = 51;
lambda_tilde = 1^(1/(Tbstar-Tb));
gamma_tilde = 1^(1/(Tbstar-Tb));

% Length of simulation
add = 12;
T = max(Tmstar,Tbstar) + add;

% IRF graphs 
from = 1;
up_to = T;
hit = Tm + 0; % Time when shock hits

% Anticipated / Unanticipated


% Final Structure
ppi_ss_f = ppi_ss*lambda^(Tmstar-Tm);
r_ss_f = ppi_ss_f + g_ss - log(beta); % Steady state nominal interest rate.
rho_pi_f = rho_pi*gamma^(Tmstar-Tm);

% Vector for evolution of target
target = zeros(1,T);
target_tilde = zeros(1,T);
inflation_response = zeros(1,T);
inflation_response_tilde = zeros(1,T);

%% Initialize Structural Matrices

A0t = zeros(n,n,T);
A1t = zeros(n,n,T);
B0t = zeros(n,n,T); 
At = zeros(n,n,T);
Bt = zeros(n,n,T);
D0t = zeros(n,nshocks,T);
Dt = zeros(n,nshocks,T);

A0tilde = zeros(n,n,T);
A1tilde = zeros(n,n,T);
B0tilde = zeros(n,n,T);
D0tilde = zeros(n,nshocks,T);
Atilde = zeros(n,n,T);
Btilde = zeros(n,n,T);
Dtilde = zeros(n,nshocks,T);

%% Initial structure 1:Tm-1 
% Set up Binder Pesaran matrices
A0 = zeros(n,n);
A1 = zeros(n,n);
B0 = zeros(n,n);
D0 = zeros(n,nshocks);
% Euler equation
% x(t) = (r_ss-ppi_ss) - (r(t) - Et pi(t+1)) + Et x(t+1) +
% (1-omega)(1-rho_a)a(t)
A0(1,r) = 1;
A0(1,a) = -(1-rho_a)*(1-omega);
A0(1,x) = 1;
A0(1,const) = ppi_ss - r_ss;
B0(1,x) = 1;
B0(1,ppi) = 1;
% Phillips Curve
% (1+beta*alpha) pi(t) = (1+beta*alpha-alpha-beta)*ppi_ss + alpha pi(t-1) + beta*Et pi(t+1) + psi x(t) - ehat(t))
A0(2,e) = 1;
A0(2,x) = -psi;
A0(2,ppi) = 1 + beta*alpha;
A0(2,const) = -(1-beta+beta*alpha-alpha)*ppi_ss;
A1(2,ppi) = alpha;
B0(2,ppi) = beta;
% Monetary Policy Rule
% r(t) = (1-rho_r)*rss -(rho_pi+rho_E)*ppi_ss - rho_g*g_ss + rho_r*r(t-1) +
% rho_pi*pi(t) + rho_E Et pi(t+1)
% + rho_g*g(t) + rho_x*x(t) + er(t)
A0(3,r) = 1;
A0(3,g) = -rho_g;
A0(3,x) = -rho_x;
A0(3,ppi) = -rho_pi;
A0(3,const) = -((1-rho_r)*r_ss-(rho_pi)*ppi_ss-rho_g*g_ss);
A1(3,r) = rho_r;
D0(3,er) = 1;
% Output gap
% x(t) = yhat(t) - omega a(t)
A0(4,yhat) = -1;
A0(4,a) = omega;
A0(4,x) = 1;
% Growth
% g(t) = g_ss + yhat(t) - yhat(t-1) + e_z(t)
A0(5,yhat) = -1;
A0(5,g) = 1;
A0(5,const) = -g_ss;
A1(5,yhat) = -1;
D0(5,ez) = 1;
% Preference shocks
% a(t) = rho_a*a(t-1) + e_a(t)
A0(6,a) = 1;
A1(6,a) = rho_a;
D0(6,ea) = 1;
% Mark-up shock
% e(t) = rho_e*e(t-1) + e_e(t)
A0(7,e) = 1;
A1(7,e) = rho_e;
D0(7,ee) = 1;
% Constant
% const(t) = const(t-1)
A0(8,const) = 1;
A1(8,const) = 1;
% AR(1)s are in y(t) so
Gamma = zeros(nshocks);
% D0 is entered above;
D2 = zeros(n, nshocks);
% Solution of Initial Structure
[Q, G, unique, rc, iter] = smatsbp(A0,A1,B0,D0,D2,Gamma);
% Steady state
P = Q(1:end-1,1:end-1);
C = Q(1:end-1,end);
y_ss = (eye(n-1)-P)\C;
y_ss(n) = 1;

for t = 1:Tm-1
    A0t(:,:,t) = A0;
    A1t(:,:,t) = A1;
    B0t(:,:,t) = B0;
    D0t(:,:,t) = D0;
    At(:,:,t) = A0t(:,:,t)\A1t(:,:,t);
    Bt(:,:,t) = A0t(:,:,t)\B0t(:,:,t);
    Dt(:,:,t) = A0t(:,:,t)\D0t(:,:,t);
    target(1,t) = ppi_ss;
    inflation_response(1,t) = rho_pi;
end

for t = 1:Tb-1
    A0tilde(:,:,t) = A0;
    A1tilde(:,:,t) = A1;
    B0tilde(:,:,t) = B0;
    D0tilde(:,:,t) = D0;
    Atilde(:,:,t) = A0tilde(:,:,t)\A1tilde(:,:,t);
    Btilde(:,:,t) = A0tilde(:,:,t)\B0tilde(:,:,t);
    Dtilde(:,:,t) = A0tilde(:,:,t)\D0tilde(:,:,t);
    target_tilde(1,t) = ppi_ss;
    inflation_response_tilde(1,t) = rho_pi;
end

%% Structures Tm:Tmstar-1 

ppi0 = ppi_ss;

for t = Tm:Tmstar-1;
    A0t(:,:,t) = A0;
    A1t(:,:,t) = A1;
    B0t(:,:,t) = B0;
    eval(['ppi_',int2str(t),'=lambda^(t-Tm+1)*ppi0;'])
    eval(['rho_pi_',int2str(t),'=gamma^(t-Tm+1)*rho_pi;'])
    r_ss = eval(['ppi_',int2str(t)]) + g_ss - log(beta); % Steady state nominal interest rate.
    % Euler equation
    % x(t) = (r_ss-ppi_ss) - (r(t) - Et pi(t+1)) + Et x(t+1) +
    % (1-omega)(1-rho_a)a(t)
    A0t(1,r,t) = 1;
    A0t(1,a,t) = -(1-rho_a)*(1-omega);
    A0t(1,x,t) = 1;
    A0t(1,const,t) = eval(['ppi_',int2str(t)]) - r_ss;
    B0t(1,x,t) = 1;
    B0t(1,ppi,t) = 1;
    % Phillips Curve
    % (1+beta*alpha) pi(t) = (1+beta*alpha-alpha-beta)*ppi_ss + alpha pi(t-1) + beta*Et pi(t+1) + psi x(t) - ehat(t))
    A0t(2,e,t) = 1;
    A0t(2,x,t) = -psi;
    A0t(2,ppi,t) = 1 + beta*alpha;
    A0t(2,const,t) = -(1-beta+beta*alpha-alpha)*eval(['ppi_',int2str(t)]);
    A1t(2,ppi,t) = alpha;
    B0t(2,ppi,t) = beta;
    % Monetary Policy Rule
    % r(t) = (1-rho_r)*rss -(rho_pi+rho_E)*ppi_ss - rho_g*g_ss + rho_r*r(t-1) +
    % rho_pi*pi(t) + rho_E Et pi(t+1)
    % + rho_g*g(t) + rho_x*x(t) + er(t)
    A0t(3,r,t) = 1;
    A0t(3,g,t) = -rho_g;
    A0t(3,x,t) = -rho_x;
    A0t(3,ppi,t) = -eval(['rho_pi_',int2str(t)]);
    A0t(3,const,t) = -((1-rho_r)*r_ss-(eval(['rho_pi_',int2str(t)]))*eval(['ppi_',int2str(t)])-rho_g*g_ss);
    A1t(3,r,t) = rho_r;
    
    At(:,:,t) = A0t(:,:,t)\A1t(:,:,t);
    Bt(:,:,t) = A0t(:,:,t)\B0t(:,:,t);
    D0t(:,:,t) = D0;
    Dt(:,:,t) = A0t(:,:,t)\D0t(:,:,t);
    
    target(1,t) = eval(['ppi_',int2str(t)]);
    inflation_response(1,t) = eval(['rho_pi_',int2str(t)]);
end

%% Tilde Structures Tb:Tbstar-1

ppi0_tilde = ppi_ss;

for t = Tb:Tbstar-1;
    A0tilde(:,:,t) = A0;
    A1tilde(:,:,t) = A1;
    B0tilde(:,:,t) = B0;
    eval(['ppi_tilde_',int2str(t),'=lambda_tilde^(t-Tb+1)*ppi0_tilde;'])
    eval(['rho_pi_tilde_',int2str(t),'=gamma_tilde^(t-Tb+1)*rho_pi;'])
    r_ss = eval(['ppi_tilde_',int2str(t)]) + g_ss - log(beta); % Steady state nominal interest rate.
    % Euler equation
    % x(t) = (r_ss-ppi_ss) - (r(t) - Et pi(t+1)) + Et x(t+1) +
    % (1-omega)(1-rho_a)a(t)
    A0tilde(1,r,t) = 1;
    A0tilde(1,a,t) = -(1-rho_a)*(1-omega);
    A0tilde(1,x,t) = 1;
    A0tilde(1,const,t) = eval(['ppi_tilde_',int2str(t)]) - r_ss; 
    B0tilde(1,x,t) = 1;
    B0tilde(1,ppi,t) = 1;
    % Phillips Curve
    % (1+beta*alpha) pi(t) = (1+beta*alpha-alpha-beta)*ppi_ss + alpha pi(t-1) + beta*Et pi(t+1) + psi x(t) - ehat(t))
    A0tilde(2,e,t) = 1;
    A0tilde(2,x,t) = -psi;
    A0tilde(2,ppi,t) = 1 + beta*alpha;
    A0tilde(2,const,t) = -(1-beta+beta*alpha-alpha)*eval(['ppi_tilde_',int2str(t)]);
    A1tilde(2,ppi,t) = alpha;
    B0tilde(2,ppi,t) = beta;
    % Monetary Policy Rule
    % r(t) = (1-rho_r)*rss -(rho_pi+rho_E)*ppi_ss - rho_g*g_ss + rho_r*r(t-1) +
    % rho_pi*pi(t) + rho_E Et pi(t+1)
    % + rho_g*g(t) + rho_x*x(t) + er(t)
    A0tilde(3,r,t) = 1;
    A0tilde(3,g,t) = -rho_g;
    A0tilde(3,x,t) = -rho_x;
    A0tilde(3,ppi,t) = -eval(['rho_pi_tilde_',int2str(t)]);
    A0tilde(3,const,t) = -((1-rho_r)*r_ss-(eval(['rho_pi_tilde_',int2str(t)]))*eval(['ppi_tilde_',int2str(t)])-rho_g*g_ss);
    A1tilde(3,r,t) = rho_r;

    Atilde(:,:,t) = A0tilde(:,:,t)\A1tilde(:,:,t);
    Btilde(:,:,t) = A0tilde(:,:,t)\B0tilde(:,:,t);
    D0tilde(:,:,t) = D0;
    Dtilde(:,:,t) = A0tilde(:,:,t)\D0tilde(:,:,t);
    
    target_tilde(1,t) = eval(['ppi_tilde_',int2str(t)]);
    inflation_response_tilde(1,t) = eval(['rho_pi_tilde_',int2str(t)]);
end;

%% Final Structure Tmstar:T 

A0f = zeros(n,n);
A1f = zeros(n,n);
B0f = zeros(n,n);
D0f = zeros(n,nshocks);
% Euler equation
% x(t) = (r_ss-ppi_ss) - (r(t) - Et pi(t+1)) + Et x(t+1) +
% (1-omega)(1-rho_a)a(t)
A0f(1,r) = 1;
A0f(1,a) = -(1-rho_a)*(1-omega);
A0f(1,x) = 1;
A0f(1,const) = ppi_ss_f - r_ss_f;
B0f(1,x) = 1;
B0f(1,ppi) = 1;
% Phillips Curve
% pi(t) = (1-beta)*ppi_ss + beta*Et pi(t+1) + psi x(t) - ehat(t))
A0f(2,e) = 1;
A0f(2,x) = -psi;
A0f(2,ppi) = 1 + beta*alpha;
A0f(2,const) = -(1-beta+beta*alpha-alpha)*ppi_ss_f;
A1f(2,ppi) = alpha;
B0f(2,ppi) = beta;
% Monetary Policy Rule
% r(t) = (1-rho_r)*rss -(rho_pi+rho_E)*ppi_ss - rho_g*g_ss + rho_r*r(t-1) +
% rho_pi*pi(t) + rho_E Et pi(t+m1)
% + rho_g*g(t) + rho_x*x(t) + er(t)
A0f(3,r) = 1;
A0f(3,g) = -rho_g;
A0f(3,x) = -rho_x;
A0f(3,ppi) = -rho_pi_f;
A0f(3,const) = -((1-rho_r)*r_ss_f-(rho_pi_f)*ppi_ss_f-rho_g*g_ss);
A1f(3,r) = rho_r;
D0f(3,er) = 1;
% Output gap
% x(t) = yhat(t) - omega a(t)
A0f(4,yhat) = -1;
A0f(4,a) = omega;
A0f(4,x) = 1;
% Growth
% g(t) = g_ss + yhat(t) - yhat(t-1) + e_z(t)
A0f(5,yhat) = -1;
A0f(5,g) = 1;
A0f(5,const) = -g_ss;
A1f(5,yhat) = -1;
D0f(5,ez) = 1;
% Preference shocks
% a(t) = rho_a*a(t-1) + e_a(t)
A0f(6,a) = 1;
A1f(6,a) = rho_a;
D0f(6,ea) = 1;
% Mark-up shock
% e(t) = rho_e*e(t-1) + e_e(t)
A0f(7,e) = 1;
A1f(7,e) = rho_a;
D0f(7,ee) = 1;
% Constant
% const(t) = const(t-1)
A0f(8,const) = 1;
A1f(8,const) = 1;
Gammaf = zeros(nshocks);
% D0 is entered above;
D2f = zeros(n, nshocks);
% Solution of Final Structure

[Qf, Gf, uniquef, rcf, iterf] = smatsbp(A0f,A1f,B0f,D0f,D2f,Gammaf);

% Steady state of final structure
Pf = Qf(1:end-1,1:end-1);
Cf = Qf(1:end-1,end);
y_ss_f = (eye(n-1)-Pf)\Cf;
y_ss_f(n) = 1;

for t = Tmstar:T
    A0t(:,:,t) = A0f;
    A1t(:,:,t) = A1f;
    B0t(:,:,t) = B0f;
    D0t(:,:,t) = D0f;
    At(:,:,t) = A0t(:,:,t)\A1t(:,:,t);
    Bt(:,:,t) = A0t(:,:,t)\B0t(:,:,t);
    Dt(:,:,t) = A0t(:,:,t)\D0t(:,:,t);
    
    target(1,t) = ppi_ss_f;
    inflation_response(1,t) = rho_pi_f;
end

for t = Tbstar:T
    A0tilde(:,:,t) = A0f;
    A1tilde(:,:,t) = A1f;
    B0tilde(:,:,t) = B0f;
    D0tilde(:,:,t) = D0f;
    Atilde(:,:,t) = A0tilde(:,:,t)\A1tilde(:,:,t);
    Btilde(:,:,t) = A0tilde(:,:,t)\B0tilde(:,:,t);
    Dtilde(:,:,t) = A0tilde(:,:,t)\D0tilde(:,:,t);
    
    target_tilde(1,t) = ppi_ss_f;
    inflation_response_tilde(1,t) = rho_pi_f;
end

%% Reduced Forms

         disp('Anticipated Changes Computation')
         % Perfect credibility
         Qt = zeros(n,n,T);
         Gt = zeros(n,nshocks,T);

         % Initial structure
         for t = 1:Tm-1
             Qt(:,:,t) = Q;
             Gt(:,:,t) = G;
         end
         % Transition
         % Start backward recursion
         Qt(:,:,Tmstar-1) = (eye(n)-Bt(:,:,Tmstar-1)*Qf)\At(:,:,Tmstar-1);
         Gt(:,:,Tmstar-1) = (eye(n)-Bt(:,:,Tmstar-1)*Qf)\Dt(:,:,Tmstar-1);
         for t = (Tmstar-2):-1:Tm
             Qt(:,:,t) = (eye(n)-Bt(:,:,t)*Qt(:,:,t+1))\At(:,:,t);
             Gt(:,:,t) = (eye(n)-Bt(:,:,t)*Qt(:,:,t+1))\Dt(:,:,t);
         end
         % Final structure
         for t = Tmstar:T
             Qt(:,:,t) = Qf;
             Gt(:,:,t) = Gf;
         end 

        % Imperfect credibility
         Qtilde = zeros(n,n,T);
         Gtilde = zeros(n,nshocks,T);
         Qhat = zeros(n,n,T);
         Ghat = zeros(n,nshocks,T);

        % Initial structure
        for t = 1:Tb-1
            Qtilde(:,:,t) = Q;
            Gtilde(:,:,t) = G;
            Qhat(:,:,t) = Q;
            Ghat(:,:,t) = G;
        end

       % Transition 
       % Start backward recursion
       Qtilde(:,:,Tbstar-1) = (eye(n)-Btilde(:,:,Tbstar-1)*Qf)\Atilde(:,:,Tbstar-1);
       Gtilde(:,:,Tbstar-1) = (eye(n)-Btilde(:,:,Tbstar-1)*Qf)\Dtilde(:,:,Tbstar-1);
       for t = (Tbstar-2):-1:Tb
           Qtilde(:,:,t) = (eye(n)-Btilde(:,:,t)*Qtilde(:,:,t+1))\Atilde(:,:,t);
           Gtilde(:,:,t) = (eye(n)-Btilde(:,:,t)*Qtilde(:,:,t+1))\Dtilde(:,:,t);
       end

       % Final structure
       for t = Tbstar:T+1
           Qtilde(:,:,t) = Qf;
           Gtilde(:,:,t) = Gf;
       end

      % Actual reduced form
       for t = 1:Tb-1
           Qhat(:,:,t) = (A0t(:,:,t)-B0t(:,:,t)*Q)\A1t(:,:,t);
           Ghat(:,:,t) = (A0t(:,:,t)-B0t(:,:,t)*Q)\D0t(:,:,t);
       end
       for t = Tb:Tbstar-1
           Qhat(:,:,t) = (A0t(:,:,t)-B0t(:,:,t)*Qtilde(:,:,t+1))\A1t(:,:,t);
           Ghat(:,:,t) = (A0t(:,:,t)-B0t(:,:,t)*Qtilde(:,:,t+1))\D0t(:,:,t);
       end
       for t = Tbstar:T
           Qhat(:,:,t) = (A0t(:,:,t)-B0t(:,:,t)*Qf)\A1t(:,:,t);
           Ghat(:,:,t) = (A0t(:,:,t)-B0t(:,:,t)*Qf)\D0t(:,:,t);
       end

    

   
%% Nonstochastic Response

% Select shock

et = zeros(nshocks,T);


y_pc = zeros(n,T); % response under perfect credibility
y_pc(:,1) = y_ss;
y_im = zeros(n,T); % response under imperfect credibility
y_im(:,1) = y_ss;

for t = 2:T
    y_pc(:,t) = Qt(:,:,t) * y_pc(:,t-1) + Gt(:,:,t) * et(:,t);
    y_im(:,t) = Qhat(:,:,t) * y_im(:,t-1) + Ghat(:,:,t) * et(:,t);
end

%% IRF during transition


et_ea = zeros(nshocks,T);

et_ea(ea,hit) = -1.5*sig_a;


% Impulse response under perfect credibility
y_pc_ea = zeros(n,T);
y_pc_ea(:,1) = y_ss;

% Impulse response under imperfect credibility
y_im_ea = y_pc_ea;

% Impulse response under initial structure
yini_ea = y_pc_ea;

% Impulse response under final structure
yfin_ea = zeros(n,T);
yfin_ea(:,1) = y_ss_f;


for t = 2:T

    y_pc_ea(:,t) = Qt(:,:,t) * y_pc_ea(:,t-1) + Gt(:,:,t) * et_ea(:,t);
    
    y_im_ea(:,t) = Qhat(:,:,t) * y_im_ea(:,t-1) + Ghat(:,:,t) * et_ea(:,t);
        
    yini_ea(:,t) = Q * yini_ea(:,t-1) + G * et_ea(:,t);
    
    yfin_ea(:,t) = Qf * yfin_ea(:,t-1) + Gf * et_ea(:,t);
    
end


%% Plots Options
t = from:up_to; % length of figure. 
lw = 1.0;





















